class message_new:
    def __init__(self, response):
        self.response = response
        self.message_object = response['message']
        self.message_date = self.message_object['date']
        self.from_id = self.message_object['from_id']
        self.message_id = self.message_object['id']
        self.out = self.message_object['out']
        self.conversation_message_id = self.message_object['conversation_message_id']
        self.forward_messages = self.message_object['fwd_messages']
        self.is_important = self.message_object['important']
        self.is_hidden = self.message_object['is_hidden']
        self.random_id = self.message_object['random_id']
        self.text = self.message_object['text']
        self.client_info = response['client_info']
        self.button_actions = self.client_info['button_actions']



import json
import random
import requests
from VKLong.object import *
from VKLong.exceptions import *


class Event:
    def __init__(self, update):
        self.type = update["updates"][0]['type']
        self.response = update['updates'][0]['object']
        if self.type == "message_new":
            self.object = message_new(self.response)
            if self.response['message']['peer_id'] < 2000000000:
                self.is_message_from_user = True
            else:
                self.is_message_from_user = False

class Bot:
    def __init__(self, token: str, api_version: int = 5.131, wait=25):
        self.token = token
        self.api_version = api_version
        self.wait = 25
        self.__api_link = "https://api.vk.com/method"
        self.__group_id = self.execute_api("groups.getById", {"access_token": token, 'v': api_version})[0]['id']
        self.__last_update = None

    # Функция вызова API:
    def execute_api(self, method, parameters):
        if type(parameters) != dict:
            raise VKLongBotExceptions.API.WrongArgumentsType("Parameters type for execute API-request needed be a dictionary!")
        else:
            parameters['access_token'] = self.token
            parameters['v'] = self.api_version
            response = requests.get(f"{self.__api_link}/{method}", params=parameters).json()
            return response['response']

    # Получение данных сервера:
    def get_actual_server_data(self):
        response = self.execute_api("groups.getLongPollServer", {"group_id": self.__group_id})
        return {"server": response['server'], "key": response['key'], "ts": response['ts']}

    # Получение обновлений:
    def get_updates(self, function):
        server_data = self.get_actual_server_data()
        server = server_data['server']
        server_ts = int(server_data['ts'])
        server_key = server_data['key']
        while True:
            response = requests.get(f"{server}?act=a_check&key={server_key}&ts={server_ts}&wait={self.wait}").json()
            try:
                error_number = int(response['failed'])
                match error_number:
                    case 1:
                        server_ts = response['ts']
                    case 2:
                        server_key = self.get_actual_server_data()['key']
                    case 3:
                        server = self.get_actual_server_data()
                        server_ts = server['ts']
                        server_key = server['key']
            except KeyError:
                try:
                    self.__last_update = response
                    function(Event(response))
                except IndexError:
                    pass
                server_ts += 1

    # Отправка сообщений:
    def send_message(self, peer_id: int or list, message: str = None, keyboard = None, attachment: str = None, random_id: int = 0,
                     reply_id: int = None, forward_messages = None, sticker_id: int = None, payload = None, dont_parse_links: int = None, disable_mentions: int = None):
        return self.execute_api("messages.send",
                         {"peer_id": peer_id, "message": message, "keyboard": keyboard, "attachment": attachment, "random_id": random_id,
                          "reply_id": reply_id, "forward_messages": forward_messages, "sticker_id": sticker_id,
                          "payload": payload, "dont_parse_links": dont_parse_links, "disable_mentions": disable_mentions})

    # Ответ на сообщение (без указания peer_id, сообщение будет отправлено последнему отправителю):

    def answer(self, message: str = None, keyboard = None, attachment: str = None, random_id: int = 0,
                     reply_id: int = None, forward_messages = None, sticker_id: int = None, payload = None, dont_parse_links: int = None, disable_mentions: int = None):
        peer_id = self.__last_update['updates'][0]['object']['message']['peer_id']
        return self.execute_api("messages.send",
                         {"peer_id": peer_id, "message": message, "keyboard": keyboard, "attachment": attachment, "random_id": random_id,
                          "reply_id": reply_id, "forward_messages": forward_messages, "sticker_id": sticker_id,
                          "payload": payload, "dont_parse_links": dont_parse_links, "disable_mentions": disable_mentions})

    # Подробнее: https://dev.vk.com/api/bots/development/keyboard
    def send_message_event_answer(self, event_id: str, user_id: int, peer_id: int, event_data: dict):
        self.execute_api("messages.sendMessageEventAnswer", {'event_id': event_id, 'user_id': user_id, 'peer_id': peer_id, 'event_data': json.dumps(event_data)})



import json

class KeyboardColor:
    PRIMARY = "primary"
    BLUE = PRIMARY
    SECONDARY = "secondary"
    WHITE = SECONDARY
    NEGATIVE = "negative"
    RED = NEGATIVE
    POSITIVE = "positive"
    GREEN = POSITIVE


class KeyboardGenerator:

    def __init__(self, one_time=True, inline=False):
        self.keyboard_json = dict()
        self.current_line = 0
        self.keyboard_json['one_time'] = one_time
        self.keyboard_json['buttons'] = []
        self.keyboard_json['inline'] = inline

    # Добавление текстовой кнопки:
    def add_text_button(self, text: str, payload: str = None, color: str = KeyboardColor.WHITE):
        if not self.keyboard_json['buttons']:
            self.keyboard_json['buttons'].append([{"action": {'type': 'text', 'label': text, 'payload': payload}, "color": color}])
        else:
            self.keyboard_json['buttons'][self.current_line].append({"action": {'type': 'text', 'label': text, 'payload': payload}, "color": color})

    # Добавление кнопки, которая откроет ссылку:
    def add_openlink_button(self, text: str, open_link: str, payload: str = None):
        if not self.keyboard_json['buttons']:
            self.keyboard_json['buttons'].append([{"action": {'type': 'open_link', 'label': text, 'link': open_link, 'payload': payload}}])
        else:
            self.keyboard_json['buttons'][self.current_line].append({"action": {'type': 'open_link', 'label': text, 'link': open_link, 'payload': payload}})

    # Добавление кнопки, которая запросит локацию у пользователя:
    def add_geolocation_button(self, payload: str = None):
        if not self.keyboard_json['buttons']:
            self.keyboard_json['buttons'].append([{"action": {'type': 'location', 'payload': payload}}])
        else:
            self.keyboard_json['buttons'][self.current_line].append({"action": {'type': 'location', 'payload': payload}})

    # Добавление кнопки, которая перенаправит на окно оплаты VK Pay:
    def add_open_vkpay_form_button(self, payment_hash: str, payload: str = None):
        if not self.keyboard_json['buttons']:
            self.keyboard_json['buttons'].append([{"action": {'type': 'vkpay', 'hash': payment_hash,'payload': payload}}])
        else:
            self.keyboard_json['buttons'][self.current_line].append({"action": {'type': 'vkpay', 'hash': payment_hash, 'payload': payload}})

    # Добавление кнопки, которая откроет VK Mini Apps:
    def add_open_miniapps_button(self, app_id: int, app_name: str, owner_id: int = None,  payload: str = None, navigation_hash: str = None):
        if not self.keyboard_json['buttons']:
            self.keyboard_json['buttons'].append([{"action": {'type': 'open_app', 'app_id': app_id, 'app_name': app_name, 'owner_id': owner_id, 'hash': navigation_hash, 'payload': payload}}])
        else:
            self.keyboard_json['buttons'][self.current_line].append({"action": {'type': 'open_app', 'app_id': app_id, 'app_name': app_name, 'owner_id': owner_id, 'hash': navigation_hash, 'payload': payload}})

    # Добавление Callback-кнопки:
    def add_callback_button(self, text: str, payload: str = None):
        if not self.keyboard_json['buttons']:
            self.keyboard_json['buttons'].append([{"action": {'type': 'callback', 'label': text, 'payload': payload}}])
        else:
            self.keyboard_json['buttons'][self.current_line].append({"action": {'type': 'callback', 'label': text, 'payload': payload}})

    def add_new_line(self):
        self.current_line += 1
        self.keyboard_json['buttons'].append([])

    def get_keyboard_json(self):
        return json.dumps(self.keyboard_json)



class VKLongBotExceptions (Exception):
    # Ошибки использования API:
    class API (Exception):
        class WrongArgumentsType (Exception):
            def __init__(self, error_text):
                pass


from setuptools import setup

setup(
    name="VKLong",
    version="0.2.0.2",
    description="Simple wrapper to make VK-Bot on Longpoll API",
    install_requires=[
        'requests'
     ],
     author="LeetCrash",
     author_email="leetcrash.official@gmail.com",
     keywords=[
        "vk api",
        "vk api library",
        "vk longpoll",
        "vk bot",
        "vklong",
        "vkapi",
        "vk long poll bot"
     ],
     zip_safe=False
)